//----------------------------------------------------------------------------
//Kionix 3axis accelerometer KXP84-AKImodule
//
//P0[1]:X_Analog
//P0[3]:Y_Analog
//P0[5]:Z_Analog
//
//P1[2]:SPIM_MISO
//P1[3]:ChipSelect
//P1[4]:SPIM_MOSI
//P1[6]:SPIM_SCLK
//P2[7]:SPIM_RESET
//
//P2[0-6]:LCD
//
//
//----------------------------------------------------------------------------

#include <m8c.h>        // part specific constants and macros
#include "PSoCAPI.h"    // PSoC API definitions for all User Modules

unsigned int AnalogData[3]={0,0,0};
unsigned int counter = 0;
unsigned char loopcounter = 0;
unsigned char i=0;
unsigned int DigitalData[3] = {0,0,0};
unsigned int SPI_Buffer = 0;
unsigned char sts;

/*KXP84 Register*/
unsigned char CTRL_REGA_READ  = 0x8C;
unsigned char CTRL_REGB_WRITE = 0x0B;
unsigned char CTRL_REGB_READ  = 0x8B;
unsigned char CTRL_REGC_WRITE = 0x0A;
unsigned char CTRL_REGC_READ  = 0x8A;
 
unsigned char FF_INT_WRITE = 0x06;
unsigned char FF_INT_READ  = 0x86;
unsigned char FF_DELAY_WRITE = 0x07;
unsigned char FF_DELAY_READ  = 0x87;
unsigned char MOT_INT_WRITE = 0x08;
unsigned char MOT_INT_READ  = 0x88;
unsigned char MOT_DELAY_WRITE = 0x09;
unsigned char MOT_DELAY_READ  = 0x89;

unsigned char WRITE_ADDS[6] = {0x0B,0x0A,0x06,0x07,0x08,0x09};//REGB,REGC,FF_INT,FF_DELAY,MOT_INT,MOT_DELAY
unsigned char WRITE_DATS[6] = {0x06,0x00,0x14,0x14,0x4D,0x14};
unsigned char READ_ADDS[6]  = {0x8B,0x8A,0x86,0x87,0x88,0x89};//REGB,REGC,FF_INT,FF_DELAY,MOT_INT,MOT_DELAY
unsigned char AMUX_REGS[3] = {0x01,0x03,0x05};

unsigned char XOUT_H = 0x80;
unsigned char XOUT_L = 0x81;
unsigned char YOUT_H = 0x82;
unsigned char YOUT_L = 0x83;
unsigned char ZOUT_H = 0x84;
unsigned char ZOUT_L = 0x85;

unsigned char OUT_H = 0x80;

void main()
{
    /*Start Modules*/
	/*ChipSelect High*/
	
	PRT1DR |= 0b00001000;
	//SclkDriver_Start();
	M8C_EnableGInt;
	PGA_Start(0x02);
	ADCINC_Start(0x02);
	AMUX4_Start();
	LCD_Start();
	LCD_Init();
	LCD_Position(0,0);
	LCD_PrCString("SPI:");
	LCD_Position(1,0);
	LCD_PrCString("ADC:");
	SPIM_Start(SPIM_SPIM_MODE_0|SPIM_SPIM_MSB_FIRST);//SPI MODE0 and MSB First
	/*KXP84 Start Up*/
	/*Reset*/
	PRT2DR  |= 0b10000000;//RESET PIN  (P2[7])HIGH
	PRT2DR  &= 0b01111111;//RESET PIM  (P2[7])LOW
	PRT2DM2 &= 0b01111111;//RESET PIN Pull Down
	PRT2DM1 &= 0b01111111;//
	PRT2DM0 &= 0b01111111;//
	/*Reset Done*/
	
	
	/*CTRL_REGB*/
	for(loopcounter = 0;loopcounter<7;loopcounter++)//test while@timing measurement
	{
		/*Write Section*/
		/*Write Address*/
		PRT1DR &= 0b11110111;//ChipSelect Low
		while(!(SPIM_bReadStatus() & SPIM_SPIM_TX_BUFFER_EMPTY)); 
		SPIM_SendTxData(WRITE_ADDS[loopcounter]);
		SPIM_SendTxData(WRITE_DATS[loopcounter]);
		while(!(SPIM_bReadStatus() & SPIM_SPIM_SPI_COMPLETE));
		PRT1DR |= 0b00001000;//ChipSelect High

	}

	PRT1DR &= 0b11110111;//ChipSelect Low
	PRT1DR |= 0b00001000;//ChipSelect High

	for(counter=0;counter<1;counter++);
	
	/*End Start Up Condition*/
	
	/*Run Code*/
	while(1)
	{	
		/*SPI Section*/
		
		for(counter = 0;counter<3;counter++)
		{
			PRT1DR &= 0b11110111;//ChipSelect Low
			/*Select axis data*/
			OUT_H |= counter*2;
			/*Send Buffer*/
			while( ! (SPIM_bReadStatus() & SPIM_SPIM_TX_BUFFER_EMPTY ) );    
    		SPIM_SendTxData(OUT_H);//OUT_H
			while(!(SPIM_bReadStatus() & SPIM_SPIM_SPI_COMPLETE)); 
			/*Wait 200 ns(minimum)*/
			/*Read Buffer*/
			SPIM_SendTxData(0x00);//Dammy
			//SPIM_SendTxData(0x00);//Dammy
			do 
			{
				sts = bSPIM_ReadStatus();
			}
			while (!(sts & SPIM_SPIM_RX_BUFFER_FULL));
			SPI_Buffer = SPIM_bReadRxData();
			SPI_Buffer = SPI_Buffer<<8;
			
			while( ! (SPIM_bReadStatus() & SPIM_SPIM_TX_BUFFER_EMPTY ) );    
    		SPIM_SendTxData(OUT_H+1);//OUT_H
			while(!(SPIM_bReadStatus() & SPIM_SPIM_SPI_COMPLETE)); 
			/*Wait 200 ns(minimum)*/
			/*Read Buffer*/
			SPIM_SendTxData(0x00);//Dammy
			//SPIM_SendTxData(0x00);//Dammy
			do 
			{
				sts = bSPIM_ReadStatus();
			}
			while (!(sts & SPIM_SPIM_RX_BUFFER_FULL));
			SPI_Buffer |= SPIM_bReadRxData();
			SPI_Buffer = SPI_Buffer>>4;
			DigitalData[counter] = SPI_Buffer;
			PRT1DR |= 0b00001000;//ChipSelect High
			OUT_H = 0x80;
		}
		/*ADC Section*/
		for(counter = 0;counter<3;counter++)
		{
			AMUX4_InputSelect(counter);
			ADCINC_GetSamples(1);
			while(ADCINC_fIsDataAvailable() == 0);
			AnalogData[counter] = ADCINC_wClearFlagGetData();
		}
	
		LCD_Position(0,4);
		LCD_PrHexInt(DigitalData[0]);
		LCD_Position(0,8);
		LCD_PrHexInt(DigitalData[1]);
		LCD_Position(0,12);
		LCD_PrHexInt(DigitalData[2]);
		LCD_Position(1,4);
		LCD_PrHexInt(AnalogData[0]);
		LCD_Position(1,8);
		LCD_PrHexInt(AnalogData[1]);
		LCD_Position(1,12);
		LCD_PrHexInt(AnalogData[2]);
		OUT_H = 0x80;				
	}//while loop end
}//main end